/*
 * Copyright 2019 by Kappich Systemberatung Aachen
 * ALL RIGHTS RESERVED.
 */

package de.bsvrz.pat.sysbed.plugins.api;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.LinkedListMultimap;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.authentication.AuthenticationFile;
import de.bsvrz.dav.daf.main.authentication.ClientCredentials;
import de.bsvrz.dav.daf.main.authentication.InteractiveGuiAuthentication;
import de.bsvrz.dav.daf.main.authentication.UserProperties;
import de.bsvrz.pat.sysbed.plugins.api.settings.SettingsData;
import de.bsvrz.sys.funclib.consoleProcessFrame.ConsoleProcessFrame;
import de.bsvrz.sys.funclib.consoleProcessFrame.ConsoleProcessPanel;

import javax.swing.JOptionPane;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.function.Supplier;

/**
 * Basisklasse für Module, die einen externen Java-Prozess (DAF-Standardapplikation) starten. Diese Klasse unterstützt dabei, Datenverteiler, Benutzer, Authentifizierung usw. zu erledigen.
 * für die 
 *
 * @since 3.14
 * @author Kappich Systemberatung
 */
public abstract class ExternalProcessModule extends ExternalModuleAdapter {

	private ConsoleProcessFrame _javaProcessFrame;

	/**
	 * Gibt die Standardargumente zurück. Aktuell ist dies "-benutzer", "-authentifizierung", und "-datenverteiler".
	 * Es können in Zukunft weitere Argumente ergänzt werden.
	 *
	 * Diese Methode kann in Ausnahmefällen überschrieben werden, um eigene Werte zu setzen.
	 * 
	 * @return Standardargumente
	 */
	protected LinkedHashMap<String, String> getStandardArguments() {
		LinkedHashMap<String, String> result = new LinkedHashMap<>();
		ClientDavInterface connection = getConnection();
		String userName = connection.getLocalUser().getName();
		ClientDavParameters clientDavParameters = connection.getClientDavParameters();
		result.put("-benutzer", userName);
		result.put("-authentifizierung", "STDINDIRECT");
		result.put("-datenverteiler", clientDavParameters.getDavCommunicationAddress() + ":" + clientDavParameters.getDavCommunicationSubAddress());
		return result;
	}

	private Supplier<char[]> getPasswordSupplier() {
		UserProperties userProperties = getConnection().getClientDavParameters().getUserProperties();
		String name = getConnection().getLocalUser().getName();
		if(!(userProperties instanceof AuthenticationFile)) {
			userProperties = InteractiveGuiAuthentication.getInstance(_javaProcessFrame);
		}
		final UserProperties finalUserProperties = userProperties;
		return () -> {
			ClientCredentials clientCredentials = finalUserProperties.getClientCredentials(name);
			if(clientCredentials == null) {
				return null;
			}
			return clientCredentials.toString().toCharArray();
		};
	}

	/**
	 * Startet einen Java-Prozess.
	 * @param settingsData Einstellungen dieses Moduls. Mit desen Einstellungen wird {@link #getParameterList(SettingsData)} aufgerufen.
	 * @param mainClass    Java-Main-Klasse des zu startenden Prozesses
	 * @param title        Titel des Fensters
	 */
	public void startProcess(final SettingsData settingsData, final Class<?> mainClass, final String title) {
		final LinkedListMultimap<String, String> parameterList = getParameterList(settingsData);
		for(Map.Entry<String, String> entry : getStandardArguments().entrySet()) {
			parameterList.replaceValues(entry.getKey(), ImmutableList.of(entry.getValue()));
		}
		
		String[] arguments = parameterList.entries().stream().map(ExternalProcessModule::joinEntry).toArray(String[]::new);
		try {
			ConsoleProcessPanel panel = ConsoleProcessPanel.createJavaProcessPanel(mainClass.getName(), arguments, null, null);
			_javaProcessFrame = ConsoleProcessFrame.createConsoleProcessFrame(title, panel);
			panel.setPasswordSupplier(getPasswordSupplier(), "\u21a0");
			panel.start();
		}
		catch(IOException ex) {
			String message = ex.getMessage();
			JOptionPane.showMessageDialog(getApplication().getParent(), message, "Fehler beim Starten von \"" + title + "\"", JOptionPane.ERROR_MESSAGE);
		}
	}

	private static String joinEntry(final Map.Entry<String, String> entry) {
		if(entry.getValue() == null) return entry.getKey();
		return entry.getKey() + "=" + entry.getValue();
	}

	/**
	 * Sammelt alle Einstellungen in einer Liste und gibt sie zurück. Die Liste enthält die Parameter für den Datengenerator. Einstellungen, die auch in {@link #getStandardArguments()} erzeugt werden, werden ignoriert. Um diese zu überschreiben muss {@link #getStandardArguments()} überschrieben werden.
	 * <p>
	 *     Dieses Verhalten ist aus Gründen der Kompatibilität erforderlich, damit alte Einstellungen nicht
	 *     den Verbindungsaufbau behindern, o. ä.
	 * </p>
	 *
	 * @param settingsData Einstellungsdaten
	 *
	 * @return Liste aller Einstellungen dieses Dialogs
	 */
	protected abstract LinkedListMultimap<String, String> getParameterList(SettingsData settingsData);
}
